'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fontoxpath = require('fontoxpath');

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var Registry = /*#__PURE__*/function () {
  /**
   * A class that you instantiate to contain "metadata" associated with certain XML nodes. The metadata could be anything,
   * but in context of being an "xml renderer" you'll probably want to use it for templates or React components.
   *
   * See also {@link GenericRenderer} and {@link ReactRenderer} which extend the `Registry` class and add a `.render()`
   * method to it.
   *
   * Render functions (metadata) are associated with XML nodes via an XPath test. For any given node, the renderer will
   * use the metadata associated the most specific test that matches the node.
   */
  function Registry() {
    /**
     * All test/value sets known to this registery. Is kept in descending order of test specificity because {@link
     * Registry.optimize} is always called when modifying this set through public methods.
     */
    this.sets = [];
    this.merge.apply(this, arguments);
  }
  /**
   * Reorders the sets based on XPath test specificity so that an `Array#find` finds the closest matching test
   * first.
   *
   * For example `<b />` could match tests `self::b` as well as `self::b[not(child::*)]`, but the latter is more
   * specific so it wins.
   *
   * This method is private because it is already called at all relevant times. Calling it again will normally not
   * yield any different results.
   */


  var _proto = Registry.prototype;

  _proto.optimize = function optimize() {
    this.sets = this.sets // Sort alphabetically by test to get a consistent sorting even if selectors are equally specific
    .sort(function (setLeft, setRight) {
      return setLeft.test.localeCompare(setRight.test);
    }) // Sort by descreasing specificity as determined by fontoxpath
    .sort(function (setLeft, setRight) {
      return fontoxpath.compareSpecificity(setRight.test, setLeft.test);
    });
  };

  /**
   * Merges other registry instances into this one, and optimizes ({@link Registry.optimize}) when done.
   */
  _proto.merge = function merge() {
    for (var _len = arguments.length, sets = new Array(_len), _key = 0; _key < _len; _key++) {
      sets[_key] = arguments[_key];
    }

    this.sets = sets.reduce(function (sets, registry) {
      return sets // Remove any duplicates from the pre-existing set
      .filter(function (set) {
        return !registry.sets.some(function (s) {
          return s.test === set.test;
        });
      }).concat(registry.sets);
    }, this.sets);
    this.optimize();
  }
  /**
   * Add a test/value set to the registry, and optimizes ({@link Registry.optimize}).
   */
  ;

  _proto.add = function add(test, value) {
    if (value === undefined) {
      throw new TypeError('Required to pass a value when adding to registry.');
    }

    if (this.sets.some(function (set) {
      return set.test === test;
    })) {
      throw new TypeError('Refusing to add a selector in duplicate, use #overwrite() instead.');
    }

    this.sets.push({
      test: test,
      value: value
    });
    this.optimize();
  };

  _proto.overwrite = function overwrite(test, value) {
    if (value === undefined) {
      throw new TypeError('Required to pass a value when overwriting to registry, use #remove() instead.');
    }

    var index = this.sets.findIndex(function (set) {
      return set.test === test;
    });

    if (index < 0) {
      throw new TypeError('Refusing to overwrite a selector because it was never set before.');
    }

    this.sets.splice(index, 1, {
      test: test,
      value: value
    });
  }
  /**
   * Remove a test/value set from the registry. This is the opposite of the {@link Registry.add} method.
   */
  ;

  _proto.remove = function remove(test) {
    var index = this.sets.findIndex(function (set) {
      return set.test === test;
    });

    if (index < 0) {
      return false;
    }

    return this.sets.splice(index, 1).length === 1;
  }
  /**
   * Retrieve the metadata that was associated with this node before. If there are several rules that match, `.find`
   * gives you only the value of the best match.
   */
  ;

  _proto.find = function find(node) {
    var set = this.sets.find(function (set) {
      return fontoxpath.evaluateXPathToBoolean(set.test, node);
    });
    return set === null || set === void 0 ? void 0 : set.value;
  };

  _createClass(Registry, [{
    key: "length",
    get: function get() {
      return this.sets.length;
    }
  }]);

  return Registry;
}();

function traverseRenderer(registry, factory, node) {
  for (var _len = arguments.length, rest = new Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
    rest[_key - 3] = arguments[_key];
  }

  var value = registry.find(node);
  var props = {
    node: node,
    traverse: function traverse(query) {
      if (query === void 0) {
        query = './node()';
      }

      return fontoxpath.evaluateXPathToNodes(query, node).map(function (n) {
        return traverseRenderer.apply(void 0, [registry, factory, n].concat(rest));
      });
    }
  };
  return factory.apply(void 0, [value, props].concat(rest));
}
/**
 * A renderer that makes no presumptions on the type of output you want. Using the `factory` argument of {@link
 * GenericRenderer.render} it can be made to output anything, such as a long concatenated string or maybe JSONML.
 *
 * This is the more generic sibling of {@link ReactRenderer}.
 */

var GenericRenderer = /*#__PURE__*/function (_Registry) {
  _inheritsLoose(GenericRenderer, _Registry);

  function GenericRenderer() {
    return _Registry.apply(this, arguments) || this;
  }

  var _proto = GenericRenderer.prototype;

  _proto.render = function render(factory, node) {
    for (var _len2 = arguments.length, rest = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
      rest[_key2 - 2] = arguments[_key2];
    }

    return traverseRenderer.apply(void 0, [this, factory, node].concat(rest));
  };

  return GenericRenderer;
}(Registry);

/**
 * Utility function to generate a unique key for a given node. Is automatically used as the `key` prop in case you're
 * using {@link Registry.createReactRenderer}, but you may find it useful in other scenarios too.
 */
function getKeyForNode(node, identifierAttribute) {
  if (identifierAttribute === void 0) {
    identifierAttribute = 'id';
  }

  var pieces = [];
  var contextNode = node;

  while (contextNode && contextNode.parentNode) {
    // If any contextNode has an identifier, assume it to be unique and stop traversing up
    if (identifierAttribute && contextNode.nodeType === 1 && contextNode.hasAttribute && contextNode.getAttribute && contextNode.hasAttribute(identifierAttribute)) {
      pieces.push(contextNode.getAttribute(identifierAttribute));
      break;
    }

    pieces.push(Array.prototype.indexOf.call(contextNode.parentNode.childNodes, contextNode));
    contextNode = contextNode.parentNode;
  }

  return 'node-key--' + pieces.reverse().join('-');
}

/**
 * The thing that you give to {@link ReactRenderer.add} should be a React component class or function component. It is
 * given two props by xml-renderer automatically, `node` and `traverse`; see also {@link XmlRendererReactProps}.
 */
// export type ElementType<XmlRendererReactProps & P> = ElementType<XmlRendererReactProps & P>;

/**
 *
 * This is the React-specific sibling of {@link GenericRenderer}.
 */

var ReactRenderer = /*#__PURE__*/function (_Registry) {
  _inheritsLoose(ReactRenderer, _Registry);

  function ReactRenderer() {
    return _Registry.apply(this, arguments) || this;
  }

  var _proto = ReactRenderer.prototype;

  _proto.render = function render(createElement, node, additionalProps) {
    return traverseRenderer(this, function (Component, props) {
      return Component ? createElement(Component, Object.assign(props, additionalProps, {
        key: getKeyForNode(props.node)
      })) : // Returning null appears to conflict with what this factory function is supposed to do; return
      // XmlRendererReactOutput or null if no component was found.
      //
      // Returning null is not allowed for ElementType
      null;
    }, node);
  };

  return ReactRenderer;
}(Registry);

exports.GenericRenderer = GenericRenderer;
exports.ReactRenderer = ReactRenderer;
exports.default = Registry;
exports.getKeyForNode = getKeyForNode;
exports.traverseRenderer = traverseRenderer;
//# sourceMappingURL=xml-renderer.cjs.development.js.map
